use crate::canvas::ImageHandle;
use std::sync::Arc;
use crate::compu_sprite::CompuSprite;
use crate::compu_buffer::{CompuBufferHandle, CompuKernelHandle};

pub struct CompuFrame {
    // Vec<(Buffer, Kernel)>
    pure_compute: Vec<(
        Arc<CompuBufferHandle>,
        Arc<CompuKernelHandle>)>,

    // Vec<(Buffer, Image, Kernel)>
    swapped_to_image: Vec<(
        Arc<CompuBufferHandle>,
        Arc<ImageHandle>,
        Arc<CompuKernelHandle>)>,

    // Vec<(Input Buffer, Output Buffer, Kernel)>
    swapped_to_buffer: Vec<(
        Arc<CompuBufferHandle>,
        Arc<CompuBufferHandle>,
        Arc<CompuKernelHandle>)>,
}

impl CompuFrame {
    pub fn new() -> CompuFrame {
        CompuFrame {
            pure_compute: vec![],
            swapped_to_image: vec![],
            swapped_to_buffer: vec![],
        }
    }

    pub fn add(&mut self, buffer: Arc<CompuBufferHandle>, kernel: Arc<CompuKernelHandle>) {
        self.pure_compute.push((buffer, kernel));
    }

    /*
        INPUT_BUFFER -> input -> kernel -> output
                           v------------------^
        OUTPUT_BUFFER -> input X kernel X output
    */
    pub fn add_chained(&mut self,
                       input_buffer: Arc<CompuBufferHandle>,
                       output_buffer: Arc<CompuBufferHandle>,
                       kernel: Arc<CompuKernelHandle>) {
        self.swapped_to_buffer.push((input_buffer, output_buffer, kernel));
    }

    pub fn add_with_image_swap(&mut self,
                               buffer: Arc<CompuBufferHandle>,
                               kernel: Arc<CompuKernelHandle>,
                               sprite: &CompuSprite) {
        self.swapped_to_image.push((buffer, sprite.get_image_handle().clone(), kernel))
    }
}